package de.diller_technologies.filestorage_tests;

import android.Manifest;
import android.content.DialogInterface;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Environment;
import android.support.annotation.NonNull;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

public class MainActivity extends AppCompatActivity {
    private static final String TAG = "MainActivity";
    private static final int PERMISSION_REQUEST_CODE_EXTERNAL_STORAGE_WRITE = 1;

    private Button mBtnWriteFiles;
    private Button mBtnDeleteFiles;
    private TextView mTxtConsole;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        mBtnWriteFiles = (Button) findViewById(R.id.btnWriteFiles);
        mBtnDeleteFiles = (Button) findViewById(R.id.btnDeleteFiles);
        mTxtConsole = (TextView) findViewById(R.id.txtConsole);

        mBtnWriteFiles.setOnClickListener(mWriteFilesOnClickListener);
        mBtnDeleteFiles.setOnClickListener(mDeleteFilesOnClickListener);
    }

    private View.OnClickListener mWriteFilesOnClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            logMessage("WRITE TEST FILES");

            // 1) Check external storage write permission
            boolean extPermissionGranted = checkExternalWritePermission();
            if (extPermissionGranted) {
                logMessage("1) External storage write permission granted");
            } else {
                logMessage("1) External storage write permission not granted yet");
            }

            // 2) Check external storage state
            String extStorageState = Environment.getExternalStorageState();
            if(extStorageState.equals(Environment.MEDIA_MOUNTED)){
                // External Storage write- and readable
            }else if(extStorageState.equals(Environment.MEDIA_MOUNTED_READ_ONLY)){
                // External Storage only readable
            }else{
                // External Storage not available for file access
            }


            logMessage("2) extStorageState = " + extStorageState);

            // 3) Write to different Paths
            logMessage("3) Calling: getInternalStoragePath()");
            String path = getInternalStoragePath();
            writeFile(path, "I'm a file on the internal storage.");

            logMessage("4) Calling: getExternalStoragePathRoot()");
            path = getExternalStoragePathRoot();
            writeFile(path, "I'm a file on the root of the external storage.");

            logMessage("5) Calling: getExternalStoragePathAppPrivate()");
            path = getExternalStoragePathAppPrivate();
            writeFile(path, "I'm a file on the app's private directory of the external storage.");

            logMessage("6) Calling: getExternalStoragePathPublicDownloads()");
            path = getExternalStoragePathPublicDownloads();
            writeFile(path, "I'm a file on the public downloads directory of the external storage.");

        }
    };

    private void writeFile(String path, String content) {
        // a) Log path
        logMessage(String.format(" a) path = %s", path));

        // b) Add subfolder
        path += "/Textfiles";

        // c) Create file (with path to directory)
        File directory = new File(path);
        if (directory == null) {
            logMessage(" c) directory = null");
        } else {
            logMessage(" c) directory successfully obtained");
        }

        // d) Create directory
        boolean directoriesCreated = directory.mkdirs();
        if (directoriesCreated) {
            logMessage(" d) Directory created successfully");
        } else {
            logMessage(" d) Directory not created");
        }

        // e) Add filenames
        path += "/test.txt";

        // f) Create files (with path to files)
        File txtFile = new File(path);
        if (txtFile == null) {
            logMessage(" f) txtFile = null");
        } else {
            logMessage(" f) txtFile successfully obtained");
        }

        // g) Write into files
        logMessage(" g) Calling: writeFile() on txtFile");
        writeFile(txtFile, content);

        // h) Read from files
        String readTextFromInternal = readFile(txtFile);
        if (readTextFromInternal != null) {
            logMessage(" h) txtFile successfully read");
            logMessage(String.format(" h) read text = %s", readTextFromInternal));
        } else {
            logMessage(" h) txtFile read failed");
        }
    }

    private View.OnClickListener mDeleteFilesOnClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            logMessage("DELETE TEST FILES");

            logMessage("1) Calling: getInternalStoragePath()");
            String path = getInternalStoragePath();
            deleteFileFromPath(path);

            logMessage("2) Calling: getExternalStoragePathRoot()");
            path = getExternalStoragePathRoot();
            deleteFileFromPath(path);

            logMessage("3) Calling: getExternalStoragePathAppPrivate()");
            path = getExternalStoragePathAppPrivate();
            deleteFileFromPath(path);

            logMessage("4) Calling: getExternalStoragePathPublicDownloads()");
            path = getExternalStoragePathPublicDownloads();
            deleteFileFromPath(path);
        }
    };

    private void deleteFileFromPath(String path) {
        // a) Add subfolder and filename
        path += "/Textfiles/test.txt";

        // b) Create file
        File txtFile = new File(path);
        if (txtFile == null) {
            logMessage(" b) txtFile = null");
        } else {
            logMessage(" b) txtFile successfully obtained");
        }

        // c) Delete files
        if (txtFile.exists()) {
            boolean internalStorageFileDeleted = txtFile.delete();
            if (internalStorageFileDeleted) {
                logMessage(" c) txtFile deleted successfully");
            } else {
                logMessage(" c) txtFile delete failed");
            }
        } else {
            logMessage(" c) txtFile does not exist");
        }
    }

    private String getInternalStoragePath() {
        return this.getFilesDir().getAbsolutePath();
    }

    private String getExternalStoragePathRoot() {
        String externalStoragePath = null;

        File externalFilesDir = Environment.getExternalStorageDirectory();
        if (externalFilesDir != null) {
            externalStoragePath = externalFilesDir.getAbsolutePath();
        }

        return externalStoragePath;
    }

    private String getExternalStoragePathAppPrivate() {
        String externalStoragePath = null;

        File externalFilesDir = this.getExternalFilesDir(null);
        if (externalFilesDir != null) {
            externalStoragePath = externalFilesDir.getAbsolutePath();
        }

        return externalStoragePath;
    }

    private String getExternalStoragePathPublicDownloads() {
        String externalStoragePath = null;

        File externalFilesDir = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS);
        if (externalFilesDir != null) {
            externalStoragePath = externalFilesDir.getAbsolutePath();
        }

        return externalStoragePath;
    }

    private boolean writeFile(File file, String msg) {
        boolean success = false;

        FileOutputStream fileOutputStream;
        try {
            fileOutputStream = new FileOutputStream(file, true);
        } catch (FileNotFoundException e) {
            logMessage("FileNotFoundException");
            fileOutputStream = null;
        }

        if (fileOutputStream != null) {
            OutputStreamWriter outputStreamWriter = new OutputStreamWriter(fileOutputStream);

            if (outputStreamWriter != null) {
                try {
                    outputStreamWriter.append(msg);
                    success = true;
                } catch (IOException e) {
                    logMessage("IOException while writing to file");
                }

                try {
                    outputStreamWriter.close();
                    fileOutputStream.flush();
                    fileOutputStream.close();
                } catch (IOException e) {
                    logMessage("IOException while closing output stream/writer");
                }
            } else {
                logMessage("outputStreamWriter is null");
            }
        }

        return success;
    }

    private String readFile(File file) {
        String msgBuffer = null;

        FileInputStream fileInputStream;
        try {
            fileInputStream = new FileInputStream(file);
        } catch (FileNotFoundException e) {
            logMessage("FileNotFoundException");
            fileInputStream = null;
        }

        if (fileInputStream != null) {
            InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream);

            if (inputStreamReader != null) {
                BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
                if (bufferedReader != null) {
                    String line;
                    do {
                        try {
                            line = bufferedReader.readLine();
                        } catch (IOException e) {
                            line = null;
                            logMessage("IOException while reading from file");
                        }

                        if (line != null) {
                            if (msgBuffer == null) {
                                msgBuffer = line + "\n";
                            } else {
                                msgBuffer += line + "\n";
                            }
                        }
                    } while (line != null);
                }

                try {
                    bufferedReader.close();
                    inputStreamReader.close();
                    fileInputStream.close();
                } catch (IOException e) {
                    logMessage("IOException while closing input streams/reader");
                }
            } else {
                logMessage("inputStreamReader is null");
            }
        }

        return msgBuffer;
    }

    private boolean checkExternalWritePermission() {
        boolean permissionGranted = false;

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {     // SDK_INT >= 23
            int permissionCheckResult = ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE);
            if (permissionCheckResult == PackageManager.PERMISSION_GRANTED) {
                permissionGranted = true;
            } else {
                if (shouldShowRequestPermissionRationale(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                    // user denied permission request before -> show info
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);
                    dialogBuilder.setMessage("This app needs permission to save data (demo text files) " +
                            "on your device. It will not access your personal data outside of this app. " +
                            "Android will ask you to allow this app to access files on your device. " +
                            "This app will only work if you grant the access.");
                    dialogBuilder.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            requestExternalStoragePermission();
                        }
                    });
                    dialogBuilder.setTitle("Storage Permission Info");
                    dialogBuilder.setIcon(android.R.drawable.ic_dialog_alert);
                    dialogBuilder.show();
                } else {
                    requestExternalStoragePermission();
                }
            }
        } else {
            permissionGranted = true;
        }

        return permissionGranted;
    }

    private void requestExternalStoragePermission() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
                    PERMISSION_REQUEST_CODE_EXTERNAL_STORAGE_WRITE);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        switch (requestCode) {
            case PERMISSION_REQUEST_CODE_EXTERNAL_STORAGE_WRITE:
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    logMessage("onRequestPermissionsResult: EXTERNAL_STORAGE_WRITE permission granted");
                } else {
                    logMessage("onRequestPermissionsResult: EXTERNAL_STORAGE_WRITE permission denied");
                }
                break;
            default:

                break;
        }
    }

    private void logMessage(String msg) {
        mTxtConsole.append(msg + "\n");
        Log.d(TAG, msg);
    }
}
